<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="2.0"
                xmlns:html="http://www.w3.org/TR/REC-html40"
                xmlns:image="http://www.google.com/schemas/sitemap-image/1.1"
                xmlns:sitemap="http://www.sitemaps.org/schemas/sitemap/0.9"
                xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
    <xsl:output method="html" version="1.0" encoding="UTF-8" indent="yes"/>
    <xsl:template match="/">
        <html xmlns="http://www.w3.org/1999/xhtml">
            <head>
                <title>XML Sitemap</title>
                <meta http-equiv="Content-Type" content="text/html; charset=utf-8"/>
                <meta name="viewport" content="width=device-width, initial-scale=1"/>
                <style type="text/css">
                    body {
                    font-family: Helvetica, Arial, sans-serif;
                    font-size: 1rem;
                    line-height: 1.5rem;
                    text-align: center;
                    margin: 2rem 0;
                    color: #1a202c;
                    }

                    #main {
                    margin: 0 auto;
                    width: 980px;
                    max-width: 98%;
                    }

                    p a {
                    color: #b91c1c;
                    font-weight: 700;
                    }
                    p a:hover {
                    color: #b91c1c;
                    text-decoration: underline;
                    }
                    p a:visited {
                    color: #b91c1c;
                    }

                    a {
                    text-decoration: none;
                    color: #1a202c;
                    }
                    a:visited {
                    color: #2d3748;
                    }
                    a:hover {
                    text-decoration: underline;
                    }

                    table {
                    width: 100%;
                    border: none;
                    border-collapse: collapse;
                    margin-top: 2rem;
                    }

                    th {
                    text-align:left;
                    padding:1rem 2rem;
                    font-size: 1.125rem;
                    line-height: 1.75rem;
                    color: #1a202c;
                    background: #baa1e4;
                    }
                    thead th:first-child {
                    border-radius: 0.25rem 0 0 0;
                    }
                    thead th:last-child {
                    border-radius: 0 0.25rem 0 0;
                    }
                    tr.url td {
                    word-break: break-all;
                    padding: 1rem;
                    font-size: 0.875rem;
                    line-height: 1.25rem;
                    background-color: #edf2f7;
                    }
                    details {
                    padding: 0.5rem 0 0.5rem 2rem;
                    font-size: 0.75rem;
                    line-height: 1rem;
                    }
                    summary:hover {
                    text-decoration: underline;
                    cursor: pointer;
                    }
                    li.img {
                    display: flex;
                    align-items: flex-start;
                    word-break: break-all;
                    }
                    li.img span {
                    width: 100px;
                    text-align: center;
                    margin-right: 1rem;
                    }
                    li.img img {
                    max-width: 100px;
                    max-height: 100px;
                    }
                    .py-2 {
                    padding: 0.5rem 0;
                    }
                    .center {
                    text-align: center;
                    }
                    .font-bold {
                    font-weight: 700;
                    }
                </style>
            </head>
            <body>
                <div id="main">
                    <h1>XML Sitemap</h1>
                    <p>
                        This XML Sitemap was generated by
                        <a href="https://weeblr.com/4seo" target="_blank" rel="noopener noreferrer">
                            4SEO
                        </a>
                    </p>
                    <xsl:if test="count(sitemap:sitemapindex/sitemap:sitemap) &gt; 0">
                        <p class="expl">
                            It is an index file listing
                            <strong>
                                <xsl:value-of select="count(sitemap:sitemapindex/sitemap:sitemap)"/>
                            </strong>
                            sitemap files. <strong>Click on any link</strong> to view the corresponding sitemap file
                            content.
                        </p>
                        <table id="sitemap" cellpadding="3">
                            <thead>
                                <tr>
                                    <th width="75%">Sitemap</th>
                                    <th width="25%" class="center">Last Modified</th>
                                </tr>
                            </thead>
                            <tbody>
                                <xsl:for-each select="sitemap:sitemapindex/sitemap:sitemap">
                                    <xsl:variable name="sitemapURL">
                                        <xsl:value-of select="sitemap:loc"/>
                                    </xsl:variable>
                                    <tr class="url">
                                        <td>
                                            <a class="font-bold" href="{$sitemapURL}">
                                                <xsl:value-of select="sitemap:loc"/>
                                            </a>
                                        </td>
                                        <td class="center">
                                            <xsl:value-of
                                                    select="concat(substring(sitemap:lastmod,0,11),concat(' ', substring(sitemap:lastmod,12,5)),concat(' ', substring(sitemap:lastmod,20,6)))"/>
                                        </td>
                                    </tr>
                                </xsl:for-each>
                            </tbody>
                        </table>
                    </xsl:if>
                    <xsl:if test="count(sitemap:sitemapindex/sitemap:sitemap) &lt; 1">
                        <p class="expl">
                            It lists
                            <strong>
                                <xsl:value-of select="count(sitemap:urlset/sitemap:url)"/>
                            </strong>
                            URL(s) and
                            <strong>
                                <xsl:value-of select="count(sitemap:urlset/sitemap:url/image:image/image:loc)"/>
                            </strong>
                            images,
                            of which
                            <strong>
                                <xsl:value-of
                                        select="count(sitemap:urlset/sitemap:url/image:image/image:loc[not(.=preceding::*)])"/>
                            </strong>
                            are unique.
                            <br/>
                            <strong>Click on any link</strong>
                            to view the corresponding website page or image.
                        </p>
                        <table id="sitemap" cellpadding="3">
                            <thead>
                                <tr>
                                    <th colspan="2">URL</th>
                                    <th width="5%" title="Images count">Images</th>
                                    <th width="20%" title="Last Modification Time" class="center">Last Mod.</th>
                                </tr>
                            </thead>
                            <tbody>
                                <xsl:for-each select="sitemap:urlset/sitemap:url">
                                    <tr class="url">
                                        <td colspan="2">
                                            <xsl:variable name="itemURL">
                                                <xsl:value-of select="sitemap:loc"/>
                                            </xsl:variable>
                                            <a class="font-bold" href="{$itemURL}">
                                                <xsl:value-of select="sitemap:loc"/>
                                            </a>
                                        </td>
                                        <td class="center">
                                            <xsl:value-of select="count(image:image)"/>
                                        </td>
                                        <td class="center">
                                            <xsl:value-of
                                                    select="concat(substring(sitemap:lastmod,0,11),concat(' ', substring(sitemap:lastmod,12,5)),concat(' ', substring(sitemap:lastmod,20,6)))"/>
                                        </td>
                                    </tr>
                                    <xsl:if test="count(image:image) &gt; 0">
                                        <td colspan="4">
                                            <details>
                                                <summary>View images on this page</summary>
                                                <xsl:for-each select="image:image">
                                                    <ul>
                                                        <li class="img">
                                                            <xsl:variable name="imgURL">
                                                                <xsl:value-of select="image:loc"/>
                                                            </xsl:variable>
                                                            <span>
                                                                <img src="{$imgURL}"></img>
                                                            </span>
                                                            <a href="{$imgURL}" target="_blank" rel="nopener">
                                                                <xsl:value-of select="image:loc"/>
                                                            </a>
                                                        </li>
                                                    </ul>
                                                </xsl:for-each>
                                            </details>
                                        </td>
                                    </xsl:if>
                                    <xsl:if test="count(image:image) &lt; 1">
                                        <td colspan="4" class="py-2"></td>
                                    </xsl:if>
                                </xsl:for-each>
                            </tbody>
                        </table>
                    </xsl:if>
                </div>
            </body>
        </html>
    </xsl:template>
</xsl:stylesheet>
