<?php
/**
 * Project: 4SEO
 *
 * @package          4SEO
 * @copyright        Copyright Weeblr llc - 2020-2023
 * @author           Yannick Gaultier - Weeblr llc
 * @license          GNU General Public License version 3; see LICENSE.md
 * @version          4.5.1.2042
 * @date        2023-02-15
 */

namespace Weeblr\Forseo\Platform\Helpers;

use Joomla\CMS\Factory;
use Joomla\CMS\Event\AbstractEvent;
use Joomla\CMS\Event\ErrorEvent;

use Joomla\CMS\Router\Exception\RouteNotFoundException;

use Weeblr\Wblib\Forseo\Base;

// no direct access
defined('_JEXEC') || defined('WBLIB_EXEC') || die;

/**
 * Helper to check if a redirect exists for the current requested URL.
 */
class Comredirect extends Base\Base
{
	/**
	 * Check if current request matches a stored redirect and
	 * execute it if so, by using the native Redirect plugin.
	 *
	 * @return void
	 * @throws \Exception
	 */
	public function checkRedirects()
	{
		$exception = new RouteNotFoundException();

		if (version_compare(JVERSION, '4', 'ge'))
		{
			$app    = Factory::getApplication();
			$plugin = $app->bootPlugin('redirect', 'system');
			if (empty($plugin) || empty($plugin->params))
			{
				// plugin is disabled
				return;
			}

			$errorEvent = AbstractEvent::create(
				'onError',
				[
					'subject'     => $exception,
					'eventClass'  => ErrorEvent::class,
					'application' => $app,
				]
			);
			$plugin->handleError($errorEvent);
		}

		if (
			version_compare(JVERSION, '4', 'lt')
			&&
			class_exists(\PlgSystemRedirect::class)
		)
		{
			\PlgSystemRedirect::handleException(
				$exception
			);
		}

	}
}
