<?php
/**
 * Project: 4SEO
 *
 * @package          4SEO
 * @copyright        Copyright Weeblr llc - 2020-2023
 * @author           Yannick Gaultier - Weeblr llc
 * @license          GNU General Public License version 3; see LICENSE.md
 * @version          4.5.1.2042
 * @date        2023-02-15
 */

namespace Weeblr\Forseo\Data;

use Weeblr\Wblib\Forseo\Base;
use Weeblr\Wblib\Forseo\Wb;
use Weeblr\Wblib\Forseo\Db;
use Weeblr\Wblib\Forseo\System;

// Security check to ensure this file is being included by a parent file.
defined('_JEXEC') || defined('WBLIB_EXEC') || die;

class Sd extends Base\Base
{
	/**
	 * Primary Types
	 */
	const ARTICLE      = 'Article';
	const NEWS_ARTICLE = 'NewsArticle';
	const BLOG_POSTING = 'BlogPosting';
	const COURSE       = 'Course';
	const EVENT        = 'Event';
	const PRODUCT      = 'Product';
	const RECIPE       = 'Recipe';
	const VIDEO_OBJECT = 'VideoObject';

	// 1.3
	const FAQ_PAGE = 'FaqPage';
	const QUESTION = 'Question';
	const ANSWER   = 'Answer';

	// Page-level types
	const PAGE_LEVEL_TYPES = [
		self::ARTICLE,
		self::NEWS_ARTICLE,
		self::BLOG_POSTING,
		self::COURSE,
		self::EVENT,
		self::PRODUCT,
		self::RECIPE,
		self::FAQ_PAGE
	];

	const REQUIRED_FIELDS_PER_TYPE = [
		'Article'     => [
			'author',
			'datePublished',
			'headline',
			'image',
			'publisher'
		],
		'NewsArticle' => [
			'author',
			'datePublished',
			'headline',
			'image',
			'publisher'
		],
		'BlogPosting' => [
			'author',
			'datePublished',
			'headline',
			'image',
			'publisher'
		],
		'Course'      => [
			'description',
			'name'
		],
		'FaqPage'     => [
			'mainEntity'
		],
		'Event'       => [
			'location',
			'name',
			'startDate'
		],
		// Product required field list is dynamic.
		'Product'     => [
			Sd::class,
			'hasRequiredFieldsProduct'
		],
		'Recipe'      => [
			Sd::class,
			'hasRequiredFieldsRecipe'
		],
		'VideoObject' => [
			Sd::class,
			'hasRequiredFieldsVideoObject'
		]
	];

	// Local business is global, only on home page
	// or maybe in the future on other specific pages
	// such as contact.
	const LOCAL_BUSINESS = 'LocalBusiness';

	// Future: Job posting
	const JOB_POSTING = 'JobPosting';

	// Future: Fact Check
	const CLAIM        = 'Claim';
	const CLAIM_REVIEW = 'ClaimReview';

	// Future: How_To
	const HOW_TO           = 'HowTo';
	const HOW_TO_SECTION   = 'HowToSection';
	const HOW_TO_STEP      = 'HowToStep';
	const HOW_TO_SUPPLY    = 'HowToSupply';
	const HOW_TO_TOOL      = 'HowToTool';
	const HOW_TO_DIRECTION = 'HowToDirection';
	const HOW_TO_TIP       = 'HowToTip';

	// Future: Movie
	const MOVIE = 'Movie';

	/**
	 * Secondary types
	 */
	const DATE_TIME        = 'DateTime';
	const URL              = 'URL';
	const IMAGE_OBJECT     = 'ImageObject';
	const PERSON           = 'Person';
	const ORGANIZATION     = 'Organization';
	const CONTACT_POINT    = 'ContactPoint';
	const PLACE            = 'Place';
	const VIRTUAL_LOCATION = 'VirtualLocation';
	const OFFER            = 'Offer';
	const RATING           = 'Rating';
	const AGGREGATE_RATING = 'AggregateRating';
	const BREADCRUMB       = 'BreadcrumbList';

	// Loc
	const POSTAL_ADRESS   = 'PostalAddress';
	const GEO_COORDINATES = 'GeoCoordinates';

	// Hours & Time
	const OPENING_HOURS_SPECIFICATION = 'OpeningHoursSpecification';
	const DURATION                    = 'Duration';

	// Products && Offers
	const QUANTITATIVE_VALUE = 'QuantitativeValue';
	const REVIEW             = 'Review';

	const OFFERS_DISCONTINUED         = 'http://schema.org/Discontinued';
	const OFFERS_IN_STOCK             = 'http://schema.org/InStock';
	const OFFERS_IN_STORE_ONLY        = 'http://schema.org/InStoreOnly';
	const OFFERS_LIMITED_AVAILABILITY = 'http://schema.org/LimitedAvailability';
	const OFFERS_ONLINE_ONLY          = 'http://schema.org/OnlineOnly';
	const OFFERS_OUT_OF_STOCK         = 'http://schema.org/OutOfStock';
	const OFFERS_PRE_ORDER            = 'http://schema.org/PreOrder';
	const OFFERS_PRE_SALE             = 'http://schema.org/PreSale';
	const OFFERS_SOLD_OUT             = 'http://schema.org/SoldOut';

	const OFFER_ITEM_CONDITION       = 'http://schema.org/OfferItemCondition';
	const ITEM_NEW_CONDITION         = 'http://schema.org/NewCondition';
	const ITEM_USED_CONDITION        = 'http://schema.org/UsedCondition';
	const ITEM_DAMAGED_CONDITION     = 'http://schema.org/DamagedCondition';
	const ITEM_REFURBISHED_CONDITION = 'http://schema.org/RefurbishedCondition';


	// Events
	const PERFORMING_GROUP                  = 'http://schema.org/PerformingGroup';
	const DANCE_GROUP                       = 'http://schema.org/DanceGroup';
	const MUSIC_GROUP                       = 'http://schema.org/MusicGroup';
	const THEATER_GROUP                     = 'http://schema.org/TheaterGroup';
	const EVENT_STATUS_TYPE                 = 'EventStatusType';
	const EVENT_STATUS_CANCELLED            = 'http://schema.org/EventCancelled';
	const EVENT_MOVED_ONLINE                = 'http://schema.org/EventMovedOnline';
	const EVENT_STATUS_POSTPONED            = 'http://schema.org/EventPostponed';
	const EVENT_STATUS_RESCHEDULED          = 'http://schema.org/EventRescheduled';
	const EVENT_STATUS_SCHEDULED            = 'http://schema.org/EventScheduled';
	const EVENT_ATTENDANCE_MODE_ENUMERATION = 'EventAttendanceModeEnumeration';
	const ONLINE_EVENT_ATTENDANCE_MODE      = 'http://schema.org/OnlineEventAttendanceMode';
	const OFFLINE_EVENT_ATTENDANCE_MODE     = 'http://schema.org/OfflineEventAttendanceMode';
	const MIXED_EVENT_ATTENDANCE_MODE       = 'http://schema.org/MixedEventAttendanceMode';

	// Recipe
	const NUTRITION_INFORMATION = 'http://schema.org/NutritionInformation';

	/**
	 * Generic types
	 */
	const TEXT = 'Text';

	/**
	 * Fields type: auto is computed by us, custom has been filled in by user
	 */
	const FIELD_AUTO   = true;
	const FIELD_CUSTOM = false;

	/**
	 * Types of opening hours specifications
	 */
	const HOURS_TYPE_NONE      = 0;
	const HOURS_TYPE_WEEKDAYS  = 1;
	const HOURS_TYPE_ALWAYS    = 2;
	const HOURS_TYPE_CUSTOM    = 3;
	const HOURS_HOURS_TYPE_24  = 0;
	const HOURS_HOURS_TYPE_9_5 = 1;
	const HOURS_HOURS_TYPE_8_6 = 2;
	/**
	 * Opening hours Specification constants
	 */
	const HOURS_ALL_DAY_OPENS  = '00:00';
	const HOURS_ALL_DAY_CLOSES = '23:59';
	const HOURS_9_TO_5_OPENS   = '09:00';
	const HOURS_9_TO_5_CLOSES  = '17:00';
	const HOURS_8_TO_6_OPENS   = '08:00';
	const HOURS_8_TO_6_CLOSES  = '18:00';

	const DAYS_OF_WEEK_SHORT = [
		'mon',
		'tue',
		'wed',
		'thu',
		'fri',
		'sat',
		'sun'
	];

	const DAYS_OF_WEEK = [
		'monday',
		'tuesday',
		'wednesday',
		'thursday',
		'friday',
		'saturday',
		'sunday'
	];
	const WEEKDAYS     = [
		'monday',
		'tuesday',
		'wednesday',
		'thursday',
		'friday',
	];

	/**
	 * FAQ operation mode: no detection, detect based on CSS selectors, detect 4SEO shortcodes
	 */
	const DETECT_NONE  = 0;
	const DETECT_CSS   = 1;
	const DETECT_CODES = 2;

	/**
	 * Various specifications and requirements.
	 */
	const ARTICLE_MIN_IMAGE_WIDTH  = 696;
	const ARTICLE_MIN_IMAGE_PIXELS = 300000;

	/**
	 * Data types that can accept reviews.
	 * https://developers.google.com/search/docs/data-types/review-snippet
	 */
	const REVIEWABLE_TYPES = [
		self::COURSE,
		self::EVENT,
		self::HOW_TO,
		self::LOCAL_BUSINESS,
		self::MOVIE,
		self::PRODUCT,
		self::RECIPE,
		self::ORGANIZATION
	];

	const PRODUCT_GLOBAL_IDENTIFIER_TYPES = [
		'gtin',
		'gtin8',
		//		'gtin12',
		'gtin13',
		'gtin14',
		'mnp',
		//		'nsn',
		'isbn'
	];

	// Ref: http://schema.org/docs/full.html#LocalBusiness
	const LOCAL_BUSINESS_TYPES = [
		'AnimalShelter'               => 'AnimalShelter',
		'AutomotiveBusiness'          => [
			'AutoBodyShop',
			'AutoDealer',
			'AutoRental',
			'AutoRepair',
			'AutoWash',
			'GasStation',
			'MotorcycleDealer',
			'MotorcycleRepair'
		],
		'ChildCare'                   => 'ChildCare',
		'DryCleaningOrLaundry'        => 'DryCleaningOrLaundry',
		'EmergencyService'            => [
			'FireStation',
			'Hospital',
			'PoliceStation'
		],
		'EmploymentAgency'            => 'EmploymentAgency',
		'EntertainmentBusiness'       => [
			'AdultEntertainment',
			'AmusementPark',
			'ArtGallery',
			'Casino',
			'ComedyClub',
			'MovieTheater',
			'NightClub'
		],
		'FinancialService'            => [
			'AccountingService',
			'AutomatedTeller',
			'BankOrCreditUnion',
			'InsuranceAgency'
		],
		'FoodEstablishment'           => [
			'Bakery',
			'BarOrPub',
			'Brewery',
			'CafeOrCoffeeShop',
			'Distillery',
			'FastFoodRestaurant',
			'IceCreamShop',
			'Restaurant',
			'Winery'
		],
		'GovernmentOffice'            => [
			'PostOffice'
		],
		'HealthAndBeautyBusiness'     => [
			'BeautySalon',
			'DaySpa',
			'HairSalon',
			'HealthClub',
			'NailSalon',
			'TattooParlor'
		],
		'HomeAndConstructionBusiness' => [
			'Electrician',
			'GeneralContractor',
			'HVACBusiness',
			'HousePainter',
			'Locksmith',
			'MovingCompany',
			'Plumber',
			'RoofingContractor'
		],
		'InternetCafe'                => 'InternetCafe',
		'LegalService'                => [
			'Attorney',
			'Notary'
		],
		'Library'                     => 'Library',
		'LocalBusiness'               => 'LocalBusiness',
		'LodgingBusiness'             => [
			'BedAndBreakfast',
			'Campground',
			'Hostel',
			'Hotel',
			'Motel'
		],
		'MedicalBusiness'             => [
			'Dentist',
			'Optician',
			'Pharmacy',
			'Physician'
		],
		'ProfessionalService'         => 'ProfessionalService',
		'RadioStation'                => 'RadioStation',
		'RealEstateAgent'             => 'RealEstateAgent',
		'RecyclingCenter'             => 'RecyclingCenter',
		'SelfStorage'                 => 'SelfStorage',
		'ShoppingCenter'              => 'ShoppingCenter',
		'SportsActivityLocation'      => [
			'BowlingAlley',
			'ExerciseGym',
			'GolfCourse',
			'PublicSwimmingPool',
			'SkiResort',
			'SportsClub',
			'StadiumOrArena',
			'TennisComplex'
		],
		'Store'                       => [
			'AutoPartsStore',
			'BikeStore',
			'BookStore',
			'ClothingStore',
			'ComputerStore',
			'ConvenienceStore',
			'DepartmentStore',
			'ElectronicsStore',
			'Florist',
			'FurnitureStore',
			'GardenStore',
			'GroceryStore',
			'HardwareStore',
			'HobbyShop',
			'HomeGoodsStore',
			'JewelryStore',
			'LiquorStore',
			'MensClothingStore',
			'MobilePhoneStore',
			'MovieRentalStore',
			'MusicStore',
			'OfficeEquipmentStore',
			'OutletStore',
			'PawnShop',
			'PetStore',
			'ShoeStore',
			'SportingGoodsStore',
			'TireShop',
			'ToyStore',
			'WholesaleStore'
		],
		'TelevisionStation'           => 'TelevisionStation',
		'TouristInformationCenter'    => 'TouristInformationCenter',
		'TravelAgency'                => 'TravelAgency'
	];


	// NB: When using this, remember to include LocalBusiness (defined above)
	// Not included to avoid repetition.
	const ORGANIZATION_TYPES = [
		'Airline',
		'Consortium',
		'Corporation',
		'EducationalOrganization' => [
			'CollegeOrUniversity',
			'ElementarySchool',
			'HighSchool',
			'MiddleSchool',
			'Preschool',
			'School'
		],
		'FundingScheme',
		'GovernmentOrganization',
		'LibrarySystem',
		'MedicalOrganization'     => [
			'Dentist',
			'DiagnosticLab',
			'Hospital',
			'MedicalClinic',
			'Pharmacy',
			'Physician',
			'VeterinaryCare',
		],
		'NGO',
		'NewsMediaOrganization',
		'PerformingGroup'         => [
			'DanceGroup',
			'MusicGroup',
			'TheaterGroup'
		],
		'Project'                 => [
			'FundingAgency',
			'ResearchProject'
		],
		'SportsOrganization'      => [
			'SportsTeam'
		],
		'WorkersUnion'
	];

	const SD_FIELDS_DEF = [
		'fields' => [
			'url'                => [
				'type' => self::URL
			],
			'headline'           => [
				'type' => self::TEXT
			],
			'description'        => [
				'type' => self::TEXT
			],
			'author'             => [
				'type' => self::TEXT
			],
			'publisher'          => [
				'type' => self::TEXT
			],
			'image'              => [
				'type' => self::IMAGE_OBJECT
			],
			'datePublished'      => [
				'type' => self::DATE_TIME
			],
			'dateModified'       => [
				'type' => self::DATE_TIME
			],
			'aggregateRating'    => [
				'type' => self::AGGREGATE_RATING
			],
			'review'             => [
				'type' => self::REVIEW
			],
			// VideoObject
			'name'               => [
				'type' => self::TEXT
			],
			'thumbnailUrl'       => [
				'type' => self::TEXT
			],
			'contentUrl'         => [
				'type' => self::TEXT
			],
			'uploadDate'         => [
				'type' => self::DATE_TIME
			],
			// LocalBusiness
			'telephone'          => [
				'type' => self::TEXT
			],
			// Course
			'provider'           => [
				'type' => self::PERSON
			],
			// FaqPage
			'question'           => [
				'type' => self::QUESTION
			],
			'answer'             => [
				'type' => self::ANSWER
			],
			// Product
			'offerItemCondition' => [
				'type' => self::OFFER_ITEM_CONDITION
			],
			// Event
			'startDate'          => [
				'type' => self::DATE_TIME
			],
			'endDate'            => [
				'type' => self::DATE_TIME
			],
			'offerValidFrom'     => [
				'type' => self::DATE_TIME
			]
		]
	];

	/**
	 * @var array Holds SD definition.
	 */
	public static $sdFieldsSpec;

	/**
	 * Checks the SD built record for a VideoObject against
	 * required fields requirements.
	 * Most types have a static list of required fields,
	 * VideoObject require a dynamic definition.
	 *
	 * @param array $sdData
	 * @return bool
	 */
	public static function hasRequiredFieldsVideoObject($sdData)
	{
		return !empty($sdData['description'])
			   &&
			   !empty($sdData['name'])
			   &&
			   !empty($sdData['thumbnailUrl'])
			   &&
			   !empty($sdData['uploadDate'])
			   &&
			   (
				   !empty($sdData['contentUrl'])
				   ||
				   !empty($sdData['embedUrl'])
				   ||
				   !empty($sdData['offers'])
			   );
	}

	/**
	 * Checks the SD built record for a Product against
	 * required fields requirements.
	 * Most types have a static list of required fields,
	 * Products require a dynamic definition.
	 *
	 * @param array $sdData
	 * @return bool
	 */
	public static function hasRequiredFieldsProduct($sdData)
	{
		return !empty($sdData['name'])
			   &&
			   (
				   !empty($sdData['review'])
				   ||
				   !empty($sdData['aggregateRating'])
				   ||
				   !empty($sdData['offers'])
			   );
	}

	/**
	 * Checks the SD built record for a Recipe against
	 * required fields requirements.
	 * Most types have a static list of required fields,
	 * Recipe require a dynamic definition.
	 *
	 * @param array $sdData
	 * @return bool
	 */
	public static function hasRequiredFieldsRecipe($sdData)
	{
		if (
			empty($sdData['name'])
			||
			empty($sdData['image'])
		)
		{
			return false;
		}

		return true;
	}
}
