<?php
/**
 * Project: 4SEO
 *
 * @package          4SEO
 * @copyright        Copyright Weeblr llc - 2020-2023
 * @author           Yannick Gaultier - Weeblr llc
 * @license          GNU General Public License version 3; see LICENSE.md
 * @version          4.5.1.2042
 * @date        2023-02-15
 */

namespace Weeblr\Forseo\Data;

use Weeblr\Forseo\Helper;

use Weeblr\Wblib\Forseo\Wb;
use Weeblr\Wblib\Forseo\Db;
use Weeblr\Wblib\Forseo\System;

// Security check to ensure this file is being included by a parent file.
defined('_JEXEC') || defined('WBLIB_EXEC') || die;

abstract class Url extends Db\Dataobject
{
	const STATUS_OK = 0;

	const DISABLED = 0;
	const ENABLED  = 1;

	const TARGET_INTERNAL = 0;
	const TARGET_EXTERNAL = 1;

	const SOURCE_INTERNAL = 0;
	const SOURCE_EXTERNAL = 1;
	const SOURCE_CRAWL    = 2;
	const SOURCE_UNKNOWN  = 0;

	const CLICK_DEPTH_NONE = -1;

	/**
	 * @var array List of max length per column.
	 */
	protected $autotrimSpec = [
		'full_url' => 2048
	];

	/**
	 * @var string[] List of data fields representing URLs, which needs to be converted to indexable, with the
	 *     corresponding indexable column name.
	 */
	protected $storageSafeColumns = [
		'full_url'       => 'url',
	];

	/**
	 * Load instance from db by searching for a given URL.
	 * The provided URL is first processed to be "indexable", ie shortened
	 * as needed and match the format of the indexable database field.
	 *
	 * @param string $searchedFullUrl
	 * @param string $fullName
	 *
	 * @return mixed
	 * @throws \Exception
	 */
	public function loadPerUrl($searchedFullUrl, $fullName = 'full_url')
	{
		return $this->loadStorageSafe(
			$searchedFullUrl,
			$fullName
		);
	}

	/**
	 * Encode full URL values into their indexable form.
	 *
	 * @param string $key
	 * @param mixed  $value
	 *
	 * @return mixed
	 */
	public function encodeValue($key, $value)
	{
		switch ($key)
		{
			case 'full_referrer':
				// referrer must be passed as HTTP header sometimes
				// HTTP client just drops empty headers so better off
				// standardizing on using / for home page referrers;
				$value = Wb\initEmpty(
					$value,
					'/'
				);

				break;
		}

		return parent::encodeValue($key, $value);
	}
}
