<?php
/**
 * Project: 4SEO
 *
 * @package          4SEO
 * @copyright        Copyright Weeblr llc - 2020-2023
 * @author           Yannick Gaultier - Weeblr llc
 * @license          GNU General Public License version 3; see LICENSE.md
 * @version          4.5.1.2042
 * @date        2023-02-15
 */

namespace Weeblr\Forseo\Model;

use Weeblr\Wblib\Forseo\System;
use Weeblr\Wblib\Forseo\Joomla\StringHelper\StringHelper;

// Security check to ensure this file is being included by a parent file.
defined('_JEXEC') || defined('WBLIB_EXEC') || die;

class Sh404sefconfig extends Config
{
	/**
	 * Load configuration from database.
	 *
	 * @return $this
	 */
	public function load()
	{
		parent::load();

		foreach ($this->detectSh404sefData() as $key => $value)
		{
			$this->set(
				$key,
				$value
			);
		}

		return $this;
	}

	/**
	 * Look for sh404SEF URLs table with custom URLs to import.
	 *
	 * @param bool $customOnly
	 * @return array
	 */
	private function detectSh404sefData($customOnly = true)
	{
		try
		{
			$sh404sefAvailable = defined('SH404SEF_IS_RUNNING');
			$forsefAvailable   = defined('4SEF_IS_RUNNING')
								 &&
								 \Forsef::isEnabled();

			if (
				!$sh404sefAvailable
				&&
				!$forsefAvailable
			)
			{
				// we require either sh404SEF or 4SEF to be running,
				// in order to convert non-SEF URLs to SEF as data
				// is stored against non-SEF in sh404SEF and against
				// SEF in 4SEO.
				return [
					'canImportMetaFromSh404sef'    => 0,
					'canImportAliasesFromSh404sef' => 0,
				];
			}

			$db = $this->factory->getThe('db');

			$metaCount = $db->count('#__sh404sef_metas');

			$aliasesCount = $db->count(
				'#__sh404sef_aliases',
				'id',
				$db->qn('type') . ' = 0 or ' . $db->qn('type') . ' = 1'
			);
		}
		catch (\Throwable $e)
		{
			$metaCount    = 0;
			$aliasesCount = 0;
		}

		return [
			'canImportMetaFromSh404sef'    => $metaCount,
			'canImportAliasesFromSh404sef' => $aliasesCount,
		];
	}
}
