<?php
/**
 * Project:                 4SEO
 *
 * @author           Yannick Gaultier - Weeblr llc
 * @copyright        Copyright Weeblr llc - 2020-2023
 * @package          4SEO
 * @license          GNU General Public License version 3; see LICENSE.md
 * @version          @build_version_full_build@
 * @date         2023-02-15
 */

namespace Weeblr\Wblib\Forseo\Html;

use Weeblr\Wblib\Forseo\Wb;
use Weeblr\Wblib\Forseo\Factory;

// Security check to ensure this file is being included by a parent file.
defined('_JEXEC') || defined('WBLIB_Forseo_ROOT_PATH') || die;

/**
 * Interface to online QRCode generator.
 *
 */
class Qrcode
{
	const ROOT_URL = 'https://qrcode.tec-it.com/API/QRCode';

	/**
	 * Build a link to an online generated QRCode
	 *
	 * @param   string  $data
	 * @param   array   $options
	 *
	 * @return string
	 */
	public static function getLink($data, $options = [])
	{
		if (empty($data))
		{
			return '';
		}

		$defaults = [
			'errorcorrection' => 'L',
			'size'            => 'small'
		];

		$options = array_merge(
			$defaults,
			$options
		);

		return self::ROOT_URL . '?data=' . urlencode($data) . '&' . \http_build_query($options);
	}

	/**
	 * Build an img tag to display an online generated QRCode
	 *
	 * @param   string  $data
	 * @param   array   $options
	 *
	 * @return string
	 */
	public static function getImageTag($data, $attributes = [], $options = [])
	{
		$link = self::getLink($data, $options);
		if (empty($link))
		{
			return '';
		}

		$size = self::getPixelSize(
			Wb\arrayGet(
				$options,
				'size',
				'small'
			)
		);

		$attributes = array_merge(
			[
				'src'    => $link,
				'width'  => $size,
				'height' => $size
			],
			$attributes
		);

		return Factory::get()->getA(Helper::class)->makeTag(
			'img',
			$attributes,
			'',
			['close' => true]
		);
	}

	/**
	 * Figure out pixel size of a QRCode based on tect-it.com convention.
	 *
	 * @param   string  $size
	 *
	 * @return int
	 */
	private static function getPixelSize($size)
	{
		switch ($size)
		{
			case 'medium':
				$pixelSize = 547;
				break;
			case 'large':
				$pixelSize = 938;
				break;
			default:
				$pixelSize = 200;
		}

		return $pixelSize;
	}
}
