<?php
/**
 * Project:                 4SEO
 *
 * @package                 4SEO
 * @copyright               Copyright Weeblr llc - 2020-2023
 * @author                  Yannick Gaultier - Weeblr llc
 * @license                 GNU General Public License version 3; see LICENSE.md
 * @version                 @build_version_full_build@
 *
 * 2023-02-15
 */

namespace Weeblr\Wblib\Forseo\System;

// Security check to ensure this file is being included by a parent file.
defined('_JEXEC') || defined('WBLIB_Forseo_ROOT_PATH') || die;

class Data
{
	const COUNTRIES_ISO3166_1_ALPHA_2 = [

		'AF' => 'Afghanistan',
		'AX' => 'Åland Islands',
		'AL' => 'Albania',
		'DZ' => 'Algeria',
		'AS' => 'American Samoa',
		'AD' => 'Andorra',
		'AO' => 'Angola',
		'AI' => 'Anguilla',
		'AQ' => 'Antarctica',
		'AG' => 'Antigua and Barbuda',
		'AR' => 'Argentina',
		'AM' => 'Armenia',
		'AW' => 'Aruba',
		'AU' => 'Australia',
		'AT' => 'Austria',
		'AZ' => 'Azerbaijan',
		'BS' => 'Bahamas',
		'BH' => 'Bahrain',
		'BD' => 'Bangladesh',
		'BB' => 'Barbados',
		'BY' => 'Belarus',
		'BE' => 'Belgium',
		'BZ' => 'Belize',
		'BJ' => 'Benin',
		'BM' => 'Bermuda',
		'BT' => 'Bhutan',
		'BO' => 'Bolivia, Plurinational State of',
		'BQ' => 'Bonaire, Sint Eustatius and Saba',
		'BA' => 'Bosnia and Herzegovina',
		'BW' => 'Botswana',
		'BV' => 'Bouvet Island',
		'BR' => 'Brazil',
		'IO' => 'British Indian Ocean Territory',
		'BN' => 'Brunei Darussalam',
		'BG' => 'Bulgaria',
		'BF' => 'Burkina Faso',
		'BI' => 'Burundi',
		'KH' => 'Cambodia',
		'CM' => 'Cameroon',
		'CA' => 'Canada',
		'CV' => 'Cape Verde',
		'KY' => 'Cayman Islands',
		'CF' => 'Central African Republic',
		'TD' => 'Chad',
		'CL' => 'Chile',
		'CN' => 'China',
		'CX' => 'Christmas Island',
		'CC' => 'Cocos (Keeling) Islands',
		'CO' => 'Colombia',
		'KM' => 'Comoros',
		'CG' => 'Congo',
		'CD' => 'Congo, the Democratic Republic of the',
		'CK' => 'Cook Islands',
		'CR' => 'Costa Rica',
		'CI' => 'Côte d\'Ivoire',
		'HR' => 'Croatia',
		'CU' => 'Cuba',
		'CW' => 'Curaçao',
		'CY' => 'Cyprus',
		'CZ' => 'Czech Republic',
		'DK' => 'Denmark',
		'DJ' => 'Djibouti',
		'DM' => 'Dominica',
		'DO' => 'Dominican Republic',
		'EC' => 'Ecuador',
		'EG' => 'Egypt',
		'SV' => 'El Salvador',
		'GQ' => 'Equatorial Guinea',
		'ER' => 'Eritrea',
		'EE' => 'Estonia',
		'ET' => 'Ethiopia',
		'FK' => 'Falkland Islands (Malvinas)',
		'FO' => 'Faroe Islands',
		'FJ' => 'Fiji',
		'FI' => 'Finland',
		'FR' => 'France',
		'GF' => 'French Guiana',
		'PF' => 'French Polynesia',
		'TF' => 'French Southern Territories',
		'GA' => 'Gabon',
		'GM' => 'Gambia',
		'GE' => 'Georgia',
		'DE' => 'Germany',
		'GH' => 'Ghana',
		'GI' => 'Gibraltar',
		'GR' => 'Greece',
		'GL' => 'Greenland',
		'GD' => 'Grenada',
		'GP' => 'Guadeloupe',
		'GU' => 'Guam',
		'GT' => 'Guatemala',
		'GG' => 'Guernsey',
		'GN' => 'Guinea',
		'GW' => 'Guinea-Bissau',
		'GY' => 'Guyana',
		'HT' => 'Haiti',
		'HM' => 'Heard Island and McDonald Islands',
		'VA' => 'Holy See (Vatican City State)',
		'HN' => 'Honduras',
		'HK' => 'Hong Kong',
		'HU' => 'Hungary',
		'IS' => 'Iceland',
		'IN' => 'India',
		'ID' => 'Indonesia',
		'IR' => 'Iran, Islamic Republic of',
		'IQ' => 'Iraq',
		'IE' => 'Ireland',
		'IM' => 'Isle of Man',
		'IL' => 'Israel',
		'IT' => 'Italy',
		'JM' => 'Jamaica',
		'JP' => 'Japan',
		'JE' => 'Jersey',
		'JO' => 'Jordan',
		'KZ' => 'Kazakhstan',
		'KE' => 'Kenya',
		'KI' => 'Kiribati',
		'KP' => 'Korea, Democratic People\'s Republic of',
		'KR' => 'Korea, Republic of',
		'KW' => 'Kuwait',
		'KG' => 'Kyrgyzstan',
		'LA' => 'Lao People\'s Democratic Republic',
		'LV' => 'Latvia',
		'LB' => 'Lebanon',
		'LS' => 'Lesotho',
		'LR' => 'Liberia',
		'LY' => 'Libya',
		'LI' => 'Liechtenstein',
		'LT' => 'Lithuania',
		'LU' => 'Luxembourg',
		'MO' => 'Macao',
		'MK' => 'Macedonia, the Former Yugoslav Republic of',
		'MG' => 'Madagascar',
		'MW' => 'Malawi',
		'MY' => 'Malaysia',
		'MV' => 'Maldives',
		'ML' => 'Mali',
		'MT' => 'Malta',
		'MH' => 'Marshall Islands',
		'MQ' => 'Martinique',
		'MR' => 'Mauritania',
		'MU' => 'Mauritius',
		'YT' => 'Mayotte',
		'MX' => 'Mexico',
		'FM' => 'Micronesia, Federated States of',
		'MD' => 'Moldova, Republic of',
		'MC' => 'Monaco',
		'MN' => 'Mongolia',
		'ME' => 'Montenegro',
		'MS' => 'Montserrat',
		'MA' => 'Morocco',
		'MZ' => 'Mozambique',
		'MM' => 'Myanmar',
		'NA' => 'Namibia',
		'NR' => 'Nauru',
		'NP' => 'Nepal',
		'NL' => 'Netherlands',
		'NC' => 'new Caledonia',
		'NZ' => 'new Zealand',
		'NI' => 'Nicaragua',
		'NE' => 'Niger',
		'NG' => 'Nigeria',
		'NU' => 'Niue',
		'NF' => 'Norfolk Island',
		'MP' => 'Northern Mariana Islands',
		'NO' => 'Norway',
		'OM' => 'Oman',
		'PK' => 'Pakistan',
		'PW' => 'Palau',
		'PS' => 'Palestine, State of',
		'PA' => 'Panama',
		'PG' => 'Papua New Guinea',
		'PY' => 'Paraguay',
		'PE' => 'Peru',
		'PH' => 'Philippines',
		'PN' => 'Pitcairn',
		'PL' => 'Poland',
		'PT' => 'Portugal',
		'PR' => 'Puerto Rico',
		'QA' => 'Qatar',
		'RE' => 'Réunion',
		'RO' => 'Romania',
		'RU' => 'Russian Federation',
		'RW' => 'Rwanda',
		'BL' => 'Saint Barthélemy',
		'SH' => 'Saint Helena, Ascension and Tristan da Cunha',
		'KN' => 'Saint Kitts and Nevis',
		'LC' => 'Saint Lucia',
		'MF' => 'Saint Martin (French part)',
		'PM' => 'Saint Pierre and Miquelon',
		'VC' => 'Saint Vincent and the Grenadines',
		'WS' => 'Samoa',
		'SM' => 'San Marino',
		'ST' => 'Sao Tome and Principe',
		'SA' => 'Saudi Arabia',
		'SN' => 'Senegal',
		'RS' => 'Serbia',
		'SC' => 'Seychelles',
		'SL' => 'Sierra Leone',
		'SG' => 'Singapore',
		'SX' => 'Sint Maarten (Dutch part)',
		'SK' => 'Slovakia',
		'SI' => 'Slovenia',
		'SB' => 'Solomon Islands',
		'SO' => 'Somalia',
		'ZA' => 'South Africa',
		'GS' => 'South Georgia and the South Sandwich Islands',
		'SS' => 'South Sudan',
		'ES' => 'Spain',
		'LK' => 'Sri Lanka',
		'SD' => 'Sudan',
		'SR' => 'Suriname',
		'SJ' => 'Svalbard and Jan Mayen',
		'SZ' => 'Swaziland',
		'SE' => 'Sweden',
		'CH' => 'Switzerland',
		'SY' => 'Syrian Arab Republic',
		'TW' => 'Taiwan, Province of China',
		'TJ' => 'Tajikistan',
		'TZ' => 'Tanzania, United Republic of',
		'TH' => 'Thailand',
		'TL' => 'Timor-Leste',
		'TG' => 'Togo',
		'TK' => 'Tokelau',
		'TO' => 'Tonga',
		'TT' => 'Trinidad and Tobago',
		'TN' => 'Tunisia',
		'TR' => 'Turkey',
		'TM' => 'Turkmenistan',
		'TC' => 'Turks and Caicos Islands',
		'TV' => 'Tuvalu',
		'UG' => 'Uganda',
		'UA' => 'Ukraine',
		'AE' => 'United Arab Emirates',
		'GB' => 'United Kingdom',
		'US' => 'United States',
		'UM' => 'United States Minor Outlying Islands',
		'UY' => 'Uruguay',
		'UZ' => 'Uzbekistan',
		'VU' => 'Vanuatu',
		'VE' => 'Venezuela, Bolivarian Republic of',
		'VN' => 'Viet Nam',
		'VG' => 'Virgin Islands, British',
		'VI' => 'Virgin Islands, U.S.',
		'WF' => 'Wallis and Futuna',
		'EH' => 'Western Sahara',
		'YE' => 'Yemen',
		'ZM' => 'Zambia',
		'ZW' => 'Zimbabwe',
	];

	const CURRENCIES_ISO_4217 = [
		'AFN' => 'AFN - Afghani',
		'EUR' => 'EUR - Euro',
		'ALL' => 'ALL - Lek',
		'DZD' => 'DZD - Algerian Dinar',
		'USD' => 'USD - US Dollar',
		'AOA' => 'AOA - Kwanza',
		'XCD' => 'XCD - East Caribbean Dollar',
		'ARS' => 'ARS - Argentine Peso',
		'AMD' => 'AMD - Armenian Dram',
		'AWG' => 'AWG - Aruban Florin',
		'AZN' => 'AZN - Azerbaijan Manat',
		'BSD' => 'BSD - Bahamian Dollar',
		'BHD' => 'BHD - Bahraini Dinar',
		'BDT' => 'BDT - Taka',
		'BBD' => 'BBD - Barbados Dollar',
		'BYN' => 'BYN - Belarusian Ruble',
		'BZD' => 'BZD - Belize Dollar',
		'XOF' => 'XOF - CFA Franc BCEAO',
		'BMD' => 'BMD - Bermudian Dollar',
		'INR' => 'INR - Indian Rupee',
		'BTN' => 'BTN - Ngultrum',
		'BOB' => 'BOB - Boliviano',
		'BOV' => 'BOV - Mvdol',
		'BAM' => 'BAM - Convertible Mark',
		'BWP' => 'BWP - Pula',
		'NOK' => 'NOK - Norwegian Krone',
		'BRL' => 'BRL - Brazilian Real',
		'BND' => 'BND - Brunei Dollar',
		'BGN' => 'BGN - Bulgarian Lev',
		'BIF' => 'BIF - Burundi Franc',
		'CVE' => 'CVE - Cabo Verde Escudo',
		'KHR' => 'KHR - Riel',
		'XAF' => 'XAF - CFA Franc BEAC',
		'CAD' => 'CAD - Canadian Dollar',
		'KYD' => 'KYD - Cayman Islands Dollar',
		'CLP' => 'CLP - Chilean Peso',
		'CLF' => 'CLF - Unidad de Fomento',
		'CNY' => 'CNY - Yuan Renminbi',
		'COP' => 'COP - Colombian Peso',
		'COU' => 'COU - Unidad de Valor Real',
		'KMF' => 'KMF - Comorian Franc ',
		'CDF' => 'CDF - Congolese Franc',
		'NZD' => 'NZD - New Zealand Dollar',
		'CRC' => 'CRC - Costa Rican Colon',
		'HRK' => 'HRK - Kuna',
		'CUP' => 'CUP - Cuban Peso',
		'CUC' => 'CUC - Peso Convertible',
		'CZK' => 'CZK - Czech Koruna',
		'DJF' => 'DJF - Djibouti Franc',
		'DOP' => 'DOP - Dominican Peso',
		'EGP' => 'EGP - Egyptian Pound',
		'AED' => 'AED - UAE Dirham',
		'ANG' => 'ANG - Netherlands Antillean Guilder',
		'CHE' => 'CHE - WIR Euro',
		'CHF' => 'CHF - Swiss Franc',
		'CHW' => 'CHW - WIR Franc',
		'DKK' => 'DKK - Danish Krone',
		'ERN' => 'ERN - Nakfa',
		'ETB' => 'ETB - Ethiopian Birr',
		'FJD' => 'FJD - Fiji Dollar',
		'FKP' => 'FKP - Falkland Islands Pound',
		'GBP' => 'GBP - Pound Sterling',
		'GEL' => 'GEL - Lari',
		'GHS' => 'GHS - Ghana Cedi',
		'GIP' => 'GIP - Gibraltar Pound',
		'GMD' => 'GMD - Dalasi',
		'GNF' => 'GNF - Guinean Franc',
		'GTQ' => 'GTQ - Quetzal',
		'GYD' => 'GYD - Guyana Dollar',
		'HKD' => 'HKD - Hong Kong Dollar',
		'HNL' => 'HNL - Lempira',
		'HTG' => 'HTG - Gourde',
		'HUF' => 'HUF - Forint',
		'IDR' => 'IDR - Rupiah',
		'ILS' => 'ILS - New Israeli Sheqel',
		'IQD' => 'IQD - Iraqi Dinar',
		'IRR' => 'IRR - Iranian Rial',
		'ISK' => 'ISK - Iceland Krona',
		'JMD' => 'JMD - Jamaican Dollar',
		'JOD' => 'JOD - Jordanian Dinar',
		'JPY' => 'JPY - Yen',
		'KES' => 'KES - Kenyan Shilling',
		'KGS' => 'KGS - Som',
		'KPW' => 'KPW - North Korean Won',
		'KRW' => 'KRW - Won',
		'KWD' => 'KWD - Kuwaiti Dinar',
		'KZT' => 'KZT - Tenge',
		'LAK' => 'LAK - Lao Kip',
		'LBP' => 'LBP - Lebanese Pound',
		'LKR' => 'LKR - Sri Lanka Rupee',
		'LRD' => 'LRD - Liberian Dollar',
		'LSL' => 'LSL - Loti',
		'LYD' => 'LYD - Libyan Dinar',
		'MAD' => 'MAD - Moroccan Dirham',
		'MDL' => 'MDL - Moldovan Leu',
		'MGA' => 'MGA - Malagasy Ariary',
		'MKD' => 'MKD - Denar',
		'MMK' => 'MMK - Kyat',
		'MNT' => 'MNT - Tugrik',
		'MOP' => 'MOP - Pataca',
		'MRU' => 'MRU - Ouguiya',
		'MUR' => 'MUR - Mauritius Rupee',
		'MVR' => 'MVR - Rufiyaa',
		'MWK' => 'MWK - Malawi Kwacha',
		'MXN' => 'MXN - Mexican Peso',
		'MXV' => 'MXV - Mexican Unidad de Inversion (UDI)',
		'MYR' => 'MYR - Malaysian Ringgit',
		'MZN' => 'MZN - Mozambique Metical',
		'NAD' => 'NAD - Namibia Dollar',
		'NGN' => 'NGN - Naira',
		'NIO' => 'NIO - Cordoba Oro',
		'NPR' => 'NPR - Nepalese Rupee',
		'OMR' => 'OMR - Rial Omani',
		'PAB' => 'PAB - Balboa',
		'PEN' => 'PEN - Sol',
		'PGK' => 'PGK - Kina',
		'PHP' => 'PHP - Philippine Peso',
		'PKR' => 'PKR - Pakistan Rupee',
		'PLN' => 'PLN - Zloty',
		'PYG' => 'PYG - Guarani',
		'QAR' => 'QAR - Qatari Rial',
		'RON' => 'RON - Romanian Leu',
		'RSD' => 'RSD - Serbian Dinar',
		'RUB' => 'RUB - Russian Ruble',
		'RWF' => 'RWF - Rwanda Franc',
		'SAR' => 'SAR - Saudi Riyal',
		'SBD' => 'SBD - Solomon Islands Dollar',
		'SCR' => 'SCR - Seychelles Rupee',
		'SDG' => 'SDG - Sudanese Pound',
		'SEK' => 'SEK - Swedish Krona',
		'SGD' => 'SGD - Singapore Dollar',
		'SHP' => 'SHP - Saint Helena Pound',
		'SLL' => 'SLL - Leone',
		'SOS' => 'SOS - Somali Shilling',
		'SRD' => 'SRD - Surinam Dollar',
		'SSP' => 'SSP - South Sudanese Pound',
		'STN' => 'STN - Dobra',
		'SVC' => 'SVC - El Salvador Colon',
		'SYP' => 'SYP - Syrian Pound',
		'SZL' => 'SZL - Lilangeni',
		'THB' => 'THB - Baht',
		'TJS' => 'TJS - Somoni',
		'TMT' => 'TMT - Turkmenistan New Manat',
		'TND' => 'TND - Tunisian Dinar',
		'TOP' => 'TOP - Pa’anga',
		'TRY' => 'TRY - Turkish Lira',
		'TTD' => 'TTD - Trinidad and Tobago Dollar',
		'TWD' => 'TWD - New Taiwan Dollar',
		'TZS' => 'TZS - Tanzanian Shilling',
		'UAH' => 'UAH - Hryvnia',
		'UGX' => 'UGX - Uganda Shilling',
		'USN' => 'USN - US Dollar (Next day)',
		'UYI' => 'UYI - Uruguay Peso en Unidades Indexadas (UI)',
		'UYU' => 'UYU - Peso Uruguayo',
		'UYW' => 'UYW - Unidad Previsional',
		'UZS' => 'UZS - Uzbekistan Sum',
		'VES' => 'VES - Bolívar Soberano',
		'VND' => 'VND - Dong',
		'VUV' => 'VUV - Vatu',
		'WST' => 'WST - Tala',
		'XAG' => 'XAG - Silver',
		'XAU' => 'XAU - Gold',
		'XBA' => 'XBA - Bond Markets Unit European Composite Unit (EURCO)',
		'XBB' => 'XBB - Bond Markets Unit European Monetary Unit (E.M.U.-6)',
		'XBC' => 'XBC - Bond Markets Unit European Unit of Account 9 (E.U.A.-9)',
		'XBD' => 'XBD - Bond Markets Unit European Unit of Account 17 (E.U.A.-17)',
		'XDR' => 'XDR - SDR (Special Drawing Right)',
		'XPD' => 'XPD - Palladium',
		'XPF' => 'XPF - CFP Franc',
		'XPT' => 'XPT - Platinum',
		'XSU' => 'XSU - Sucre',
		'XTS' => 'XTS - Codes specifically reserved for testing purposes',
		'XUA' => 'XUA - ADB Unit of Account',
		'XXX' => 'XXX - The codes assigned for transactions where no currency is involved',
		'YER' => 'YER - Yemeni Rial',
		'ZAR' => 'ZAR - Rand',
		'ZMW' => 'ZMW - Zambian Kwacha',
		'ZWL' => 'ZWL - Zimbabwe Dollar',
	];
}